@@ -29,10 +29,10 @@
 
 #include <QtCore/QTimer>
 
-#include <QtGui/QLabel>
-#include <QtGui/QCheckBox>
-#include <QtGui/QToolTip>
-#include <QtGui/QCursor>
+#include <QLabel>
+#include <QCheckBox>
+#include <QToolTip>
+#include <QCursor>
 
 #include <QtDBus/QDBusInterface>
 #include <QtDBus/QDBusReply>
@@ -55,7 +55,7 @@ static const char kWalletEntryName[] = "drkonqi_bugzilla";
 static const char kWalletEntryUsername[] = "username";
 static const char kWalletEntryPassword[] = "password";
 
-static const char konquerorKWalletEntryName[] = "https://bugs.kde.org/index.cgi#login";
+static const char konquerorKWalletEntryName[] = KDE_BUGZILLA_URL "index.cgi#login";
 static const char konquerorKWalletEntryUsername[] = "Bugzilla_login";
 static const char konquerorKWalletEntryPassword[] = "Bugzilla_password";
 
